@@ -163,6 +163,7 @@ static int __attribute__ ((__noreturn__)) usage(int opt)
         fputs(_(" -w, --lightweight         list all TID\n"), fp);
         break;
     case PKILL:
+        fputs(_(" -<sig>                    signal to send (either number or name)\n"), fp);
         fputs(_(" -H, --require-handler     match only if signal handler is present\n"), fp);
         fputs(_(" -q, --queue <value>       integer value to be sent with the signal\n"), fp);
         fputs(_(" -e, --echo                display what is killed\n"), fp);
@@ -173,7 +174,6 @@ static int __attribute__ ((__noreturn__)) usage(int opt)
         break;
 #endif
     }
-    fputs(_(" -<sig>, --signal <sig>    signal to send (either number or name)\n"), fp);
     fputs(_(" -c, --count               count of matching processes\n"), fp);
     fputs(_(" -f, --full                use full process name to match\n"), fp);
     fputs(_(" -g, --pgroup <PGID,...>   match listed process group IDs\n"), fp);
@@ -184,6 +184,7 @@ static int __attribute__ ((__noreturn__)) usage(int opt)
     fputs(_(" -O, --older <seconds>     select where older than seconds\n"), fp);
     fputs(_(" -P, --parent <PPID,...>   match only child processes of the given parent\n"), fp);
     fputs(_(" -s, --session <SID,...>   match session IDs\n"), fp);
+    fputs(_("     --signal <sig>        signal to send (either number or name)\n"), fp);
     fputs(_(" -t, --terminal <tty,...>  match by controlling terminal\n"), fp);
     fputs(_(" -u, --euid <ID,...>       match by effective IDs\n"), fp);
     fputs(_(" -U, --uid <ID,...>        match by real IDs\n"), fp);
@@ -635,6 +636,27 @@ static size_t get_arg_max(void)
     return val;
 }
 
+/*
+ * Check if we have a long simple (non-regex) match
+ * Returns true if the string:
+ *  1) is longer than 15 characters
+ *  2) Doesn't have | or [ which are used by regex
+ * This is not an exhaustive list but catches most instances
+ * It's only used to suppress the warning
+ */
+static bool is_long_match(const char *str)
+{
+    int i, len;
+
+    if (str == NULL)
+        return FALSE;
+    if (15 >= (len = strlen(str)))
+        return FALSE;
+    for (i=0; i<len; i++)
+        if (str[i] == '|' || str[i] == '[')
+            return FALSE;
+    return TRUE;
+}
 static struct el * select_procs (int *num)
 {
 #define PIDS_GETINT(e) PIDS_VAL(EU_ ## e, s_int, stack, info)
@@ -783,7 +805,7 @@ static struct el * select_procs (int *num)
 
     *num = matches;
 
-    if ((!matches) && (!opt_full) && opt_pattern && (strlen(opt_pattern) > 15))
+    if ((!matches) && (!opt_full) && is_long_match(opt_pattern))
         xwarnx(_("pattern that searches for process name longer than 15 characters will result in zero matches\n"
                  "Try `%s -f' option to match against the complete command line."),
                program_invocation_short_name);
@@ -823,7 +845,6 @@ static int pidfd_open (pid_t pid, unsigned int flags)
 static void parse_opts (int argc, char **argv)
 {
     char opts[64] = "";
-    int sig;
     int opt;
     int criteria_count = 0;
 
@@ -869,9 +890,6 @@ static void parse_opts (int argc, char **argv)
         {NULL, 0, NULL, 0}
     };
 
-    sig = signal_option(&argc, argv);
-    if (-1 < sig)
-        opt_signal = sig;
 
 #ifdef ENABLE_PIDWAIT
     if (strcmp (program_invocation_short_name, "pidwait") == 0 ||
@@ -882,7 +900,11 @@ static void parse_opts (int argc, char **argv)
 #endif
     if (strcmp (program_invocation_short_name, "pkill") == 0 ||
         strcmp (program_invocation_short_name, "lt-pkill") == 0) {
+        int sig;
         prog_mode = PKILL;
+        sig = signal_option(&argc, argv);
+        if (-1 < sig)
+            opt_signal = sig;
 	strcat (opts, "eq:");
     } else {
         strcat (opts, "lad:vw");
@@ -895,8 +917,14 @@ static void parse_opts (int argc, char **argv)
         switch (opt) {
         case SIGNAL_OPTION:
             opt_signal = signal_name_to_number (optarg);
-            if (opt_signal == -1 && isdigit (optarg[0]))
-                opt_signal = atoi (optarg);
+            if (opt_signal == -1) {
+                if (isdigit (optarg[0]))
+                    opt_signal = atoi (optarg);
+                else {
+                    fprintf(stderr, _("Unknown signal \"%s\"."), optarg);
+                    usage('?');
+                }
+            }
             break;
         case 'e':
             opt_echo = 1;
@@ -904,6 +932,7 @@ static void parse_opts (int argc, char **argv)
 /*        case 'D':   / * FreeBSD: print info about non-matches for debugging * /
  *            break; */
         case 'F':   /* FreeBSD: the arg is a file containing a PID to match */
+            free(opt_pidfile);
             opt_pidfile = xstrdup (optarg);
             ++criteria_count;
             break;
